#!/bin/bash
# cleanup-sandboxes.sh

usage()
{
    exec 2>&1

    echo "Usage: $0 [options]"
    echo "Options:"
    echo "	-c config_file"
    echo "	-d sandbox_dir"
    echo "	-h   (help)"
    echo "	-i idle_max (in days)"
    echo "	-L log_max  (in bytes)"
    echo "	-n   (dry run)"
    echo "	-S size_max (in bytes)"
    echo "	-v   (verbose)"

    exit 1
}

DIR=/var/edgwl/SandboxDir
DRYRUN=false
IDLE_MAX=15
IDLE_MIN=7
LOG_MAX=` expr  30 '*' 1000 '*' 1000`
LOG_MIN=` expr  10 '*' 1000 '*' 1000`
SIZE_MAX=`expr 100 '*' 1000 '*' 1000`
SIZE_MIN=`expr  50 '*' 1000 '*' 1000`
VERBOSE=false

f=/etc/sysconfig/lcg
test -f $f && . $f

CONF=${LCG_LOCATION:-/opt/lcg}/etc/cleanup-sandboxes.conf

for i
do
    case $i in
    -c)
	CONF=/dev/null
	break
    esac
done

test -f $CONF && . $CONF

while test $# != 0
do
    case $1 in
    -c)
	shift
	test $# = 0 && usage
	CONF=$1
	< $CONF || exit
	. $CONF
	;;
    -d)
	shift
	test $# = 0 && usage
	DIR=$1
	;;
    -h)
	usage
	;;
    -i)
	shift
	test $# = 0 && usage
	IDLE_MAX=$1
	;;
    -L)
	shift
	test $# = 0 && usage
	LOG_MAX=$1
	;;
    -n)
	DRYRUN=true
	;;
    -S)
	shift
	test $# = 0 && usage
	SIZE_MAX=$1
	;;
    -v)
	VERBOSE=true
	;;
    -*)
	echo "$0: unknown option: $1" >&2
	usage
	;;
    *)
	echo "$0: unexpected argument: $1" >&2
	usage
    esac

    shift
done

echo "0$IDLE_MAX" | egrep '^0[0-9]+$' > /dev/null || usage

if test $IDLE_MAX -lt $IDLE_MIN
then
    echo "$0: ERROR: idle time $IDLE_MAX < $IDLE_MIN" >&2
    exit 1
fi

echo "0$LOG_MAX" | egrep '^0[0-9]+$' > /dev/null || usage

if test $LOG_MAX -lt $LOG_MIN
then
    echo "$0: ERROR: max logfile size $LOG_MAX < $LOG_MIN" >&2
    exit 1
fi

echo "0$SIZE_MAX" | egrep '^0[0-9]+$' > /dev/null || usage

if test $SIZE_MAX -lt $SIZE_MIN
then
    echo "$0: ERROR: max file size $SIZE_MAX < $SIZE_MIN" >&2
    exit 1
fi

cd $DIR || exit

cwd=`/bin/pwd`

#
# safety catch against somehow ending up in a system directory
#

sysdirs='/(afs|bin|boot|dev|etc|initrd|lib|proc|root|sbin|usr)/'

if echo "$cwd" | egrep "^$sysdirs" > /dev/null
then
    echo "$0: ERROR: bad sandbox directory $cwd" >&2
    exit 1
fi

$VERBOSE && echo === Starting at `date`

if $VERBOSE
then
    LS=-ls
    PRINT='print "$ARGV[1]: truncated to $ARGV[0]\n";'
else
    LS=
    PRINT=
fi

if $DRYRUN
then
    ECHO=echo
    PERL='
	print "$ARGV[1]: to be truncated to $ARGV[0]\n";
    '
    RM='rm   '
else
    ECHO=
    PERL='
	if (truncate($ARGV[1], $ARGV[0])) {
	    '"$PRINT"'
	} else {
	    print STDERR "$ARGV[1]: $!\n";
	}
    '
    RM=rm
fi

for i in ./??
do
    find $i -depth \
	-type d \
	    -mtime +$IDLE_MAX ! -path './??'   \
		$LS -exec $ECHO rmdir {} \; -o \
	-type f \( \
	    -mtime +$IDLE_MAX \
		$LS -exec $ECHO "$RM" {} \; -o \
	    -size  +${LOG_MAX}c \(  \
		-name '*.out'    -o \
		-name '*.err'    -o \
		-name '*.log'    -o \
		-name '*.stdout' -o \
		-name '*.stderr'    \
	    \) \
		$LS -exec perl -e "$PERL"  $LOG_MAX {} \; -o \
	    -size +${SIZE_MAX}c \
		$LS -exec perl -e "$PERL" $SIZE_MAX {} \;    \
	\)
done

$VERBOSE && echo === Finished at `date`

exit 0

